#!/bin/bash -x

SCRIPTDIR=${0%/*}

# Set this to something else in order to use a specific Camino
BINPATH=$SCRIPTDIR/../../bin

PATH=${BINPATH}:${PATH}

TEST_COUNTER=1

if [ $# -gt 0 ]; then
    TEST_COUNTER=$1
fi

# Check where we are

if [[ $0 != "./tests.sh" ]]; then
    echo "Test script must be run from directory containing the script"
    exit 1
fi	

D2T="double2txt 6"
	

# Get rid of output from previous tests
rm -f ${SCRIPTDIR}/crossing/results/* ${SCRIPTDIR}/torus/results/*

# Track in single ROI and get connection probability maps 
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 or 1
track -inputmodel pico -inputfile torus/torus_06.bingham.Bdouble.gz -outputtracts voxels -randomseed 188 -seedfile torus/rois/torus_single_roi.hdr | procstreamlines -seedfile torus/rois/torus_single_roi.hdr -inputmodel voxels -outputcp -outputroot torus/results/torus_single_roi_cp_ -gzip

# Make numbering correct
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 1

# feed these to cpstats
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 2
for operation in max min median mean; do
cpstats -operation ${operation} -inputroot torus/results/torus_single_roi_cp_1_ -outputfile torus/results/torus_single_roi_cp_1_${operation} -gzip
done

# Compare results
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 3
for operation in max min median mean; do
        for ext in hdr img.gz; do
                diff -N torus/results/torus_single_roi_cp_1_${operation}.${ext} torus/expectedResults/torus_single_roi_cp_1_${operation}.${ext}
        done
done

# Output tracts as OOGL from two ROIs in one pass
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 4
track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -outputtracts raw -randomseed 188 -seedfile torus/rois/torus_multi_roi.hdr | procstreamlines -seedfile torus/rois/torus_multi_roi.hdr -outputtracts oogl -outputroot torus/results/torus_multi_roi_streamlines_

diff --brief -N torus/results/torus_multi_roi_streamlines_1.oogl torus/expectedResults/torus_multi_roi_streamlines_1.oogl
diff --brief -N torus/results/torus_multi_roi_streamlines_2.oogl torus/expectedResults/torus_multi_roi_streamlines_2.oogl


# Split by ROI
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 5

track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -outputtracts raw -randomseed 188 -seedfile torus/rois/torus_multi_roi.hdr -outputroot torus/results/torus_multi_roi_streamlines_

cat torus/results/torus_multi_roi_streamlines_1.Bfloat torus/results/torus_multi_roi_streamlines_2.Bfloat | procstreamlines  -outputtracts raw > torus/results/torus_multi_roi_combined_streamlines.Bfloat

diff -N torus/results/torus_multi_roi_combined_streamlines.Bfloat torus/expectedResults/torus_multi_roi_combined_streamlines.Bfloat


# Same for OOGL
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 6
cat torus/results/torus_multi_roi_streamlines_2.Bfloat | procstreamlines -seedfile torus/rois/torus_multi_roi.hdr -outputtracts oogl -outputroot torus/results/torus_multi_roi_streamlines_regionindex_ -regionindex 2

# test similarity
diff -N --brief torus/results/torus_multi_roi_streamlines_2.oogl torus/results/torus_multi_roi_streamlines_regionindex_2.oogl


echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 7

# Single ROI streamline tracking with waypoint
track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -randomseed 188 -seedfile torus/rois/torus_waypoints.hdr  | procstreamlines -waypointfile torus/rois/torus_waypoints.hdr -outputtracts oogl -noresample > torus/results/torus_single_roi_waypoint.oogl

diff -N --brief torus/results/torus_single_roi_waypoint.oogl torus/expectedResults/torus_single_roi_waypoint.oogl


# Single ROI streamline tracking with exclusion (discard)
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 8

track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -randomseed 188 -seedfile torus/rois/torus_single_roi.hdr | procstreamlines -exclusionfile torus/rois/torus_exclusion.hdr -outputtracts oogl > torus/results/torus_single_roi_discard_exclude.oogl

diff -N --brief torus/results/torus_single_roi_discard_exclude.oogl torus/expectedResults/torus_single_roi_discard_exclude.oogl


# Single ROI streamline tracking with exclusion (truncate)
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 9

track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -randomseed 188 -seedfile torus/rois/torus_single_roi.hdr | procstreamlines -exclusionfile torus/rois/torus_exclusion.hdr -truncateinexclusion -outputtracts oogl > torus/results/torus_single_roi_trunc_exclude.oogl

# CP maps with waypoints
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 10

track -inputmodel pico -inputfile torus/torus_06.bingham.Bdouble.gz -outputtracts voxels -randomseed 188 -seedfile torus/rois/torus_single_roi.hdr -iterations 100 | procstreamlines -iterations 100 -seedfile torus/rois/torus_single_roi.hdr -inputmodel voxels -outputsc -outputroot torus/results/torus_single_roi_waypoint_sc_ -waypointfile torus/rois/torus_waypoints.hdr -gzip

cpstats -operation max -inputroot torus/results/torus_single_roi_waypoint_sc_1_ -outputfile torus/results/torus_single_roi_waypoint_sc_max_1 -gzip

diff -N torus/results/torus_single_roi_waypoint_sc_max_1.img.gz torus/expectedResults/torus_single_roi_waypoint_sc_max_1.img.gz


# Get target CP maps from two ROIs
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 11

track -inputmodel pico -inputfile torus/torus_06.bingham.Bdouble.gz -seedfile torus/rois/torus_multi_roi.hdr -iterations 500 -randomseed 188 | procstreamlines -iterations 500 -seedfile torus/rois/torus_multi_roi.hdr -outputcp -outputroot torus/results/torus_multi_roi_target_cp_ -targetfile torus/rois/torus_targets.hdr -gzip  

for i in 1 2 ; do for j in 1 2; do 
diff -N torus/results/torus_multi_roi_target_cp_${i}_${j}_1_0.img.gz torus/expectedResults/torus_multi_roi_target_cp_${i}_${j}_1_0.img.gz
done
done

# Directional target CP

echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 12

track -inputmodel pico -inputfile torus/torus_06.bingham.Bdouble.gz -seedfile torus/rois/torus_directional_roi.hdr -iterations 500 -randomseed 188 | procstreamlines -iterations 500 -seedfile torus/rois/torus_directional_roi.hdr -outputcp -directional -1 0 0 -outputroot torus/results/torus_directional_roi_target_cp_ -targetfile torus/rois/torus_directional_targets.hdr -gzip

maxcp torus/results/torus_directional_roi_target_cp_1_ torus/results/torus_directional_roi_target_cp_1_maxcp_0 -pd 1 -targetindex 0 -gzip
maxcp torus/results/torus_directional_roi_target_cp_1_ torus/results/torus_directional_roi_target_cp_1_maxcp_1 -pd 1 -targetindex 1 -gzip   

for i in 0 1; do diff -N torus/results/torus_directional_roi_target_cp_1_maxcp_${i}.img.gz torus/expectedResults/torus_directional_roi_target_cp_1_maxcp_${i}.img.gz ; done

# Get CBS from the single ROI
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 13

track -inputmodel pico -inputfile torus/torus_06.bingham.Bdouble.gz -seedfile torus/rois/torus_directional_roi.hdr -iterations 500 -randomseed 188 | procstreamlines -iterations 500 -seedfile torus/rois/torus_directional_roi.hdr -outputcbs -directional -1 0 0 -outputroot torus/results/torus_directional_roi_target_cp_ -targetfile torus/rois/torus_directional_targets.hdr -gzip

for i in 0 1; do diff -N torus/results/torus_directional_roi_target_cp_labelsc_1_1_${i}.img.gz torus/expectedResults/torus_directional_roi_target_cp_labelsc_1_1_${i}.img.gz ; diff -N torus/results/torus_directional_roi_target_cp_labels_1_1_${i}.img.gz torus/expectedResults/torus_directional_roi_target_cp_labels_1_1_${i}.img.gz
done

# non directional cbs
track -inputmodel pico -inputfile torus/torus_06.bingham.Bdouble.gz -seedfile torus/rois/torus_single_roi.hdr -iterations 200 -randomseed 188 | procstreamlines -iterations 200 -seedfile torus/rois/torus_single_roi.hdr -outputcbs -outputsc -outputroot torus/results/torus_single_roi_cbs_ -targetfile torus/rois/torus_targets.hdr -gzip

diff -N torus/results/torus_single_roi_cbs_labels_1_1_0.img.gz torus/expectedResults/torus_single_roi_cbs_labels_1_1_0.img.gz 
diff -N torus/results/torus_single_roi_cbs_labelsc_1_1_0.img.gz torus/expectedResults/torus_single_roi_cbs_labelsc_1_1_0.img.gz

# CP with two PDs in seed
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 14
track -inputmodel multitensor -inputfile crossing/crossing_05_0505_90.multitensor.Bdouble.gz -seedfile crossing/rois/crossing_multi_roi.hdr | procstreamlines -seedfile crossing/rois/crossing_multi_roi.hdr -outputcp -outputroot crossing/results/crossing_multi_roi_cp_ -iterations 1 -gzip

diff -N crossing/results/crossing_multi_roi_cp_1_1_1.img.gz crossing/expectedResults/crossing_multi_roi_cp_1_1_1.img.gz 
diff -N crossing/results/crossing_multi_roi_cp_1_2_1.img.gz crossing/expectedResults/crossing_multi_roi_cp_1_2_1.img.gz 

diff -N crossing/results/crossing_multi_roi_cp_2_1_1.img.gz crossing/expectedResults/crossing_multi_roi_cp_2_1_1.img.gz 
diff -N crossing/results/crossing_multi_roi_cp_2_2_1.img.gz crossing/expectedResults/crossing_multi_roi_cp_2_2_1.img.gz 

diff -N crossing/results/crossing_multi_roi_cp_2_1_2.img.gz crossing/expectedResults/crossing_multi_roi_cp_2_1_2.img.gz 
diff -N crossing/results/crossing_multi_roi_cp_2_2_2.img.gz crossing/expectedResults/crossing_multi_roi_cp_2_2_2.img.gz 



# Target CP with two PDs in seed
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 15
track -inputmodel multitensor -inputfile crossing/crossing_05_0505_90.multitensor.Bdouble.gz -seedfile crossing/rois/crossing_multi_roi.hdr | procstreamlines -seedfile crossing/rois/crossing_multi_roi.hdr -outputsc -outputroot crossing/results/crossing_multi_roi_targetsc_ -iterations 1 -targetfile crossing/rois/crossing_targets.hdr -gzip

diff -N crossing/results/crossing_multi_roi_targetsc_1_1_1_0.img.gz  crossing/expectedResults/crossing_multi_roi_targetsc_1_1_1_0.img.gz 
diff -N crossing/results/crossing_multi_roi_targetsc_1_2_1_0.img.gz  crossing/expectedResults/crossing_multi_roi_targetsc_1_2_1_0.img.gz 

diff -N crossing/results/crossing_multi_roi_targetsc_2_1_1_0.img.gz  crossing/expectedResults/crossing_multi_roi_targetsc_2_1_1_0.img.gz 
diff -N crossing/results/crossing_multi_roi_targetsc_2_2_1_0.img.gz  crossing/expectedResults/crossing_multi_roi_targetsc_2_2_1_0.img.gz 

diff -N crossing/results/crossing_multi_roi_targetsc_2_1_2_0.img.gz  crossing/expectedResults/crossing_multi_roi_targetsc_2_1_2_0.img.gz 
diff -N crossing/results/crossing_multi_roi_targetsc_2_2_2_0.img.gz  crossing/expectedResults/crossing_multi_roi_targetsc_2_2_2_0.img.gz 


# CBS with two PDs in seed
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 16
track -inputmodel multitensor -inputfile crossing/crossing_05_0505_90.multitensor.Bdouble.gz -seedfile crossing/rois/crossing_multi_roi.hdr | procstreamlines -seedfile crossing/rois/crossing_multi_roi.hdr -outputcbs -outputroot crossing/results/crossing_multi_roi_ -iterations 1 -targetfile crossing/rois/crossing_targets.hdr -gzip 

diff -N crossing/results/crossing_multi_roi_labels_1_1_0.img.gz crossing/expectedResults/crossing_multi_roi_labels_1_1_0.img.gz     
diff -N crossing/results/crossing_multi_roi_labelsc_1_1_0.img.gz crossing/expectedResults/crossing_multi_roi_labelsc_1_1_0.img.gz  

diff -N crossing/results/crossing_multi_roi_labels_1_1_0.img.gz crossing/expectedResults/crossing_multi_roi_labels_1_1_0.img.gz     
diff -N crossing/results/crossing_multi_roi_labelsc_1_1_0.img.gz crossing/expectedResults/crossing_multi_roi_labelsc_1_1_0.img.gz   

diff -N crossing/results/crossing_multi_roi_labels_2_1_0.img.gz crossing/expectedResults/crossing_multi_roi_labels_2_1_0.img.gz     
diff -N crossing/results/crossing_multi_roi_labelsc_2_1_0.img.gz crossing/expectedResults/crossing_multi_roi_labelsc_2_1_0.img.gz  

diff -N crossing/results/crossing_multi_roi_labels_2_2_0.img.gz crossing/expectedResults/crossing_multi_roi_labels_2_2_0.img.gz     
diff -N crossing/results/crossing_multi_roi_labelsc_2_2_0.img.gz crossing/expectedResults/crossing_multi_roi_labelsc_2_2_0.img.gz       

# OOGL binary format
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 17
track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -randomseed 188 -seedfile torus/rois/torus_single_roi.hdr -outputtracts ooglbinary > torus/results/torus_single_roi_binary.oogl

diff -N torus/results/torus_single_roi_binary.oogl torus/expectedResults/torus_single_roi_binary.oogl


# targetprobs2txt
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 18

targetprobs2txt -inputroot torus/expectedResults/torus_multi_roi_target_cp_ -seedfile torus/rois/torus_multi_roi.hdr -targetfile torus/rois/torus_targets.hdr > torus/results/torus_multi_roi_target_cp_r1.txt

targetprobs2txt -inputroot torus/expectedResults/torus_multi_roi_target_cp_ -seedfile torus/rois/torus_multi_roi.hdr -targetfile torus/rois/torus_targets.hdr -regionindex 2 > torus/results/torus_multi_roi_target_cp_r2.txt

diff -N torus/results/torus_multi_roi_target_cp_r1.txt torus/expectedResults/torus_multi_roi_target_cp_r1.txt
diff -N torus/results/torus_multi_roi_target_cp_r2.txt torus/expectedResults/torus_multi_roi_target_cp_r2.txt

# for crossing
targetprobs2txt -inputroot crossing/expectedResults/crossing_multi_roi_targetsc_ -seedfile crossing/rois/crossing_multi_roi.hdr -targetfile crossing/rois/crossing_targets.hdr -pd 2 -regionindex 2 > crossing/results/crossing_multi_roi_target2txt.txt

diff -N crossing/results/crossing_multi_roi_target2txt.txt crossing/expectedResults/crossing_multi_roi_target2txt.txt


# directional cp

targetprobs2txt -inputroot torus/expectedResults/torus_directional_roi_target_cp_ -seedfile torus/rois/torus_directional_roi.hdr -targetfile torus/rois/torus_directional_targets.hdr -outputfile torus/results/torus_directional_targets2txt.txt

diff -N torus/results/torus_directional_targets2txt.txt torus/expectedResults/torus_directional_targets2txt.txt

# Test -seedindex 
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 19

rm -f torus/results/torus_single_roi_discard_exclude_seedindex.Bfloat

for seed in `seq 1 16`; do track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -randomseed 188 -seedfile torus/rois/torus_single_roi.hdr -seedindex $seed | procstreamlines -seedindex $seed -exclusionfile torus/rois/torus_exclusion.hdr -outputtracts raw >> torus/results/torus_single_roi_discard_exclude_seedindex.Bfloat ; done

track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -randomseed 188 -seedfile torus/rois/torus_single_roi.hdr | procstreamlines -exclusionfile torus/rois/torus_exclusion.hdr -outputtracts raw > torus/results/torus_single_roi_discard_exclude.Bfloat 

diff -N torus/results/torus_single_roi_discard_exclude_seedindex.Bfloat torus/results/torus_single_roi_discard_exclude.Bfloat  

# Test bootstrap tracking

# Generate bootstrap data

cat torus/torus_06.Bdouble.gz | gunzip -c | datasynth -inputmodel dt -snr 16 -schemefile schemefiles/M1_N30_b1000.scheme -inputdatatype double -seed 273 -outputfile torus/torus_bootstrap_1.Bfloat.gz -gzip

cat torus/torus_06.Bdouble.gz | gunzip -c | datasynth -inputmodel dt -snr 16 -schemefile schemefiles/M1_N30_b1000.scheme -inputdatatype double -seed 274 -outputfile torus/torus_bootstrap_2.Bfloat.gz -gzip

cat crossing/crossing_05_0505_90.multitensor.Bdouble.gz | gunzip -c | datasynth -inputmodel multitensor -snr 16 -schemefile schemefiles/M1_N30_b1000.scheme -gzip -inputdatatype double -seed 273 -outputfile crossing/crossing_bootstrap_1.Bfloat.gz

cat crossing/crossing_05_0505_90.multitensor.Bdouble.gz | gunzip -c | datasynth -inputmodel multitensor -snr 16 -schemefile schemefiles/M1_N30_b1000.scheme -gzip -inputdatatype double -seed 274 -outputfile crossing/crossing_bootstrap_2.Bfloat.gz


echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 20

track -inputmodel bootstrap -randomseed 39395 -inputdatatype float -bsdatafiles torus/torus_bootstrap_1.Bfloat.gz torus/torus_bootstrap_2.Bfloat.gz -schemefile schemefiles/M1_N30_b1000.scheme -inversion 1 -iterations 1000 -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 | procstreamlines -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -iterations 1000 -outputsc -outputroot torus/results/torus_bootstrap_sc_1_ -gzip

diff -N torus/results/torus_bootstrap_sc_1_1_1_1.img.gz torus/expectedResults/torus_bootstrap_sc_1_1_1_1.img.gz


# Bootstrap with crossing
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 21

track -inputmodel bootstrap -randomseed 39395 -inputdatatype float -bsdatafiles crossing/crossing_bootstrap_1.Bfloat.gz crossing/crossing_bootstrap_2.Bfloat.gz -schemefile schemefiles/M1_N30_b1000.scheme -inversion 22 -voxclassmap crossing/crossing_vc.Bint -iterations 1000 -seedpointvox 0 16 7 -datadims 40 30 15 -voxeldims 2 2 2 | procstreamlines -seedpointvox 0 16 7 -datadims 40 30 15 -voxeldims 2 2 2 -iterations 1000 -outputsc -outputroot crossing/results/crossing_bootstrap_sc_1_ -gzip

diff -N crossing/results/crossing_bootstrap_sc_1_1_1_1.img.gz crossing/expectedResults/crossing_bootstrap_sc_1_1_1_1.img.gz



echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 22

# Single ROI streamline tracking with waypoint, turn on loop checking
track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -randomseed 188 -seedfile torus/rois/torus_waypoints.hdr  | procstreamlines -truncateloops -waypointfile torus/rois/torus_waypoints.hdr -outputtracts oogl -noresample > torus/results/torus_single_roi_waypoint_looptrunc.oogl

# There are no loops, so should get the same output as without -truncateloops
diff --brief -N torus/results/torus_single_roi_waypoint_looptrunc.oogl torus/expectedResults/torus_single_roi_waypoint.oogl



# VTK streamlines
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 23

track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -randomseed 188 -seedfile torus/rois/torus_single_roi.hdr | vtkstreamlines -seedfile torus/rois/torus_single_roi.hdr > torus/results/torus_single_roi.vtk

diff -N torus/results/torus_single_roi.vtk torus/expectedResults/torus_single_roi.vtk



# Wild bootstrap

echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 24

track -inputmodel bootstrap -wildbsmodel dt -randomseed 39395 -inputdatatype float -inputfile torus/torus_bootstrap_1.Bfloat.gz -schemefile schemefiles/M1_N30_b1000.scheme -iterations 1000 -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 | procstreamlines -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -iterations 1000 -outputsc -outputroot torus/results/torus_wildbs_sc_1_ -gzip 

diff -N torus/results/torus_wildbs_sc_1_1_1_1.img.gz torus/expectedResults/torus_wildbs_sc_1_1_1_1.img.gz

# Use bgmask, anisfile
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 25

track -inputmodel bootstrap -wildbsmodel dt -randomseed 39395 -inputdatatype float -inputfile torus/torus_bootstrap_1.Bfloat.gz -schemefile schemefiles/M1_N30_b1000.scheme -iterations 1000 -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -bgmask torus/rois/torus_halfmask.hdr | procstreamlines -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -iterations 1000 -outputsc -outputroot torus/results/torus_wildbs_mask_sc_1_ -gzip

track -inputmodel bootstrap -wildbsmodel dt -randomseed 39395 -inputdatatype float -inputfile torus/torus_bootstrap_1.Bfloat.gz -schemefile schemefiles/M1_N30_b1000.scheme -iterations 1000 -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -anisthresh 1 -anisfile torus/rois/torus_halfmask.hdr | procstreamlines -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -iterations 1000 -outputsc -outputroot torus/results/torus_wildbs_anismask_sc_1_ -gzip

diff -N torus/results/torus_wildbs_mask_sc_1_1_1_1.img.gz torus/expectedResults/torus_wildbs_mask_sc_1_1_1_1.img.gz

diff -N torus/results/torus_wildbs_anismask_sc_1_1_1_1.img.gz torus/expectedResults/torus_wildbs_anismask_sc_1_1_1_1.img.gz


echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 26

track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -seed 22382 -seedfile torus/rois/torus_halfmask.hdr | procstreamlines -outputroot torus/results/torus_ -iterations 1 -seedfile torus/rois/torus_halfmask.hdr -outputacm -outputcp -gzip

diff -N torus/results/torus_acm_cp.img.gz torus/expectedResults/torus_acm_cp.img.gz



echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 27

track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -outputtracts raw -seedfile torus/rois/torus_multi_roi.hdr | vtkstreamlines -seedfile torus/rois/torus_multi_roi.hdr > torus/results/torus_multi_roi.vtk

diff -N torus/results/torus_multi_roi.vtk torus/expectedResults/torus_multi_roi.vtk


# Tract statistics

echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 28


dtfit torus/torus_bootstrap_1.Bfloat.gz schemefiles/M1_N30_b1000.scheme > torus/torus_bs_1.inv1.Bdouble

fa < torus/torus_bs_1.inv1.Bdouble > torus/torus_bs_1.fa.img

analyzeheader -datatype double -datadims 80 32 12 -voxeldims 2 2 2 > torus/torus_bs_1.fa.hdr

rm -f torus/results/tractStats.Bdouble

rm -f torus/results/tractStats_fromVoxels.Bdouble

track -inputmodel dt -inputfile torus/torus_bs_1.inv1.Bdouble -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 > torus/results/single_streamline.Bfloat

track -inputmodel dt -inputfile torus/torus_bs_1.inv1.Bdouble -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -outputtracts voxels > torus/results/single_streamline.Bshort


for i in mean var min max median meanvar sum; do 
    
     cat torus/results/single_streamline.Bfloat | tractstats -tractstat ${i} -scalarfile torus/torus_bs_1.fa.hdr  >> torus/results/tractStats.Bdouble

    cat torus/results/single_streamline.Bshort | tractstats -inputmodel voxels -tractstat ${i}  -scalarfile torus/torus_bs_1.fa.hdr  >> torus/results/tractStats_fromVoxels.Bdouble

done

cat torus/results/single_streamline.Bfloat | tractstats -tractstat length -voxeldims 2 2 2 >> torus/results/tractStats.Bdouble

cat torus/results/single_streamline.Bshort | tractstats -inputmodel voxels -tractstat length -voxeldims 2 2 2 >> torus/results/tractStats_fromVoxels.Bdouble

# diff doesn't work too well on small binary files
cat torus/results/tractStats_fromVoxels.Bdouble | $D2T > torus/results/tractStats_fromVoxels.txt

diff -N -b torus/results/tractStats_fromVoxels.txt torus/expectedResults/tractStats_fromVoxels.txt

cat torus/results/tractStats.Bdouble | $D2T > torus/results/tractStats.txt

diff -N -b torus/results/tractStats.txt torus/expectedResults/tractStats.txt


echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 29

track -inputmodel dt -inputfile torus/torus_bs_1.inv1.Bdouble  -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 | tractstatimage -scalarfile torus/torus_bs_1.fa.hdr -countintersect -tractstat mean -imagestat mean -outputroot torus/results/torus_faMean_all -gzip

diff -N torus/results/torus_faMean_all.img.gz torus/expectedResults/torus_faMean_all.img.gz


echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 30

# Test deterministic interpolation

track -inputmodel dt -inputfile torus/torus_bs_1.inv1.Bdouble -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -interpolated > torus/results/single_streamline_dt_interp.Bfloat

track -inputmodel dt -inputfile torus/torus_bs_1.inv1.Bdouble -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -interpolated vectors > torus/results/single_streamline_vec_interp.Bfloat

diff -N torus/results/single_streamline_dt_interp.Bfloat torus/expectedResults/single_streamline_dt_interp.Bfloat
diff -N torus/results/single_streamline_vec_interp.Bfloat torus/expectedResults/single_streamline_vec_interp.Bfloat



echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 31


# Test NC
track -datadims 80 32 12 -voxeldims 2 2 2 -interpolated -inputmodel pico -inputfile torus/torus_06.bingham.Bdouble.gz -randomseed 188 -seedpointvox 17 0 7 | procstreamlines -datadims 80 32 12 -voxeldims 2 2 2 -seedpointvox 17 0 7 -outputcp -outputroot torus/results/torus_single_seed_cp_ncinterp_ -gzip

diff -N torus/results/torus_single_seed_cp_ncinterp_1_1_1.img.gz torus/expectedResults/torus_single_seed_cp_ncinterp_1_1_1.img.gz


# Bayesian tracking on bootstrap torus data

echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 32

cat torus/torus_06.Bdouble.gz | gunzip -c | datasynth -inputmodel dt -snr 16 -schemefile schemefiles/bmx7.scheme -inputdatatype double -seed 273 -outputfile torus/torus_bmx7_snr16.Bfloat.gz -gzip

track -randomseed 188 -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -inputmodel bayesdirac -inputfile torus/torus_bmx7_snr16.Bfloat.gz -schemefile schemefiles/bmx7.scheme -iterations 50 -curvepriorg 1.0 | procstreamlines -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -iterations 50 -outputsc -outputroot torus/results/torus_single_seed_bayescyldt_ -gzip 

track -randomseed 188 -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -inputmodel bayesdirac -datamodel ballstick -inputfile torus/torus_bmx7_snr16.Bfloat.gz -schemefile schemefiles/bmx7.scheme -iterations 50 -curvepriork 1.0 | procstreamlines -seedpointvox 17 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -iterations 50 -outputsc -outputroot torus/results/torus_single_seed_bayesballstick_ -gzip 

diff -N torus/results/torus_single_seed_bayescyldt_1_1_1.img.gz torus/expectedResults/torus_single_seed_bayescyldt_1_1_1.img.gz

diff -N torus/results/torus_single_seed_bayesballstick_1_1_1.img.gz torus/expectedResults/torus_single_seed_bayesballstick_1_1_1.img.gz


echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 33


# Check -curvethresh, -ipthresh equivalent

track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -outputtracts raw -seedpointvox 19 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -ipthresh 0.996 > torus/results/ipThresh.Bfloat

track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -outputtracts raw -seedpointvox 19 0 7 -datadims 80 32 12 -voxeldims 2 2 2 -curvethresh 5.1264 > torus/results/curveThresh.Bfloat

diff -N torus/results/ipThresh.Bfloat torus/results/curveThresh.Bfloat


# test end points 

echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 34

track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -seedfile torus/rois/torus_endpoints_seed.hdr | procstreamlines -endpointfile torus/rois/torus_endpoints.hdr -noresample | vtkstreamlines > torus/results/endpoints.vtk

diff -N torus/results/endpoints.vtk torus/expectedResults/endpoints.vtk


echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 35

# test nifti seeds / waypoints


track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -randomseed 188 -seedfile torus/rois/torus_waypoints.nii.gz | procstreamlines -waypointfile torus/rois/torus_waypoints.nii.gz -outputtracts oogl -noresample > torus/results/torus_single_roi_waypoint_nii.oogl

diff -N --brief torus/results/torus_single_roi_waypoint_nii.oogl torus/expectedResults/torus_single_roi_waypoint.oogl


echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 36

# test MHA seeds / waypoints

track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -randomseed 188 -seedfile torus/rois/torus_waypoints.mha | procstreamlines -waypointfile torus/rois/torus_waypoints.mha -outputtracts oogl -noresample > torus/results/torus_single_roi_waypoint_mha.oogl

diff -N --brief torus/results/torus_single_roi_waypoint_mha.oogl torus/expectedResults/torus_single_roi_waypoint.oogl


echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 37

# TEND

track -inputmodel multitensor -inputfile crossing/crossing_05_0505_90.snr20.Bdouble.gz -tend -seedfile crossing/rois/crossing_multi_roi.hdr -tendg 0.6 -gzip > crossing/results/crossing_multi_roi_tend.Bfloat.gz

diff -N crossing/results/crossing_multi_roi_tend.Bfloat.gz crossing/expectedResults/crossing_multi_roi_tend.Bfloat.gz


echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 38

# tract stats, output raw values
cat torus/results/single_streamline.Bfloat | tractstats -noresample -tractstat none -scalarfile torus/torus_bs_1.fa.hdr | $D2T > torus/results/tractFA.txt

diff -N torus/results/tractFA.txt torus/expectedResults/tractFA.txt


echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 39

# negative pixel dims

track -inputmodel pico -inputfile torus/torus_06.bingham.Bdouble.gz -seedfile torus/rois/torus_multi_roi_negpixwidth.hdr -iterations 500 -randomseed 188 | procstreamlines -iterations 500 -seedfile torus/rois/torus_multi_roi_negpixwidth.hdr -outputcp -outputroot torus/results/torus_multi_roi_target_cp_npw_ -targetfile torus/rois/torus_targets_negpixwidth.hdr -gzip  

for i in 1 2 ; do 
    for j in 1 2; do 
        diff -N torus/results/torus_multi_roi_target_cp_npw_${i}_${j}_1_0.img.gz torus/expectedResults/torus_multi_roi_target_cp_${i}_${j}_1_0.img.gz
    done
done


# Test fix of bug where non-contiguous labels 

echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 40

for i in 1 2 3; do 
track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -outputtracts raw -randomseed 188 -seedfile torus/rois/torus_multi_spacedlabels_roi.hdr -regionindex $i > torus/results/torus_spaced_labels_r${i}.Bfloat

diff -N torus/results/torus_spaced_labels_r${i}.Bfloat torus/expectedResults/torus_spaced_labels_r${i}.Bfloat

done


# ACM with targets
echo "TRACKING TEST $TEST_COUNTER"; TEST_COUNTER=$(($TEST_COUNTER + 1)) # $1 + 41

track -inputmodel dt -inputfile torus/torus_06.Bdouble.gz -outputtracts raw -randomseed 188 -seedfile torus/rois/torus_waypoints.hdr | procstreamlines -outputacm -targetfile torus/rois/torus_targets.hdr -outputroot torus/results/torus_acm_targets_ -gzip

diff -N torus/results/torus_acm_targets_acm_target_sc.img.gz torus/expectedResults/torus_acm_targets_acm_target_sc.img.gz
