function [A, T] = readNiiTransform(hdrFile, useQ)
%
% [A,T] = readNiiTransform(hdrFile, useQ)
%
% A = affine matrix, either from sform or rotation from quaternion + pixdim scaling
% T = translation
%
% [x; y; z] = A * [i; j; k] + T
% 
% Args:
%     hdrFile = file to read
%     useQ = set to 1 to use the qform transformation even if the sform
%     code is greater than zero. 
% 

hdr = imaging.Nifti1Dataset(hdrFile);

hdr.readHeader();

if (useQ || (hdr.sform_code == 0) ) 
    % Get quaternion + shift
    b = hdr.quatern(1);
    c = hdr.quatern(2);
    d = hdr.quatern(3);
      
    a = sqrt(1.0-(b*b+c*c+d*d));
    
    R = [a*a+b*b-c*c-d*d 2*b*c-2*a*d  2*b*d+2*a*c;  2*b*c+2*a*d  a*a+c*c-b*b-d*d  2*c*d-2*a*b;  2*b*d-2*a*c  2*c*d+2*a*b  a*a+d*d-c*c-b*b]; 
   
    A = R;
    
    % voxel -> mm space comes from rigid orientation + scaling by pixdims, with qfac thrown in if necessary because NIfTI always stores
    % quaternians with positive determinants 
    A(1,1) = A(1,1) * hdr.pixdim(2);
    A(2,2) = A(2,2) * hdr.pixdim(3);
    A(3,3) = A(3,3) * hdr.qfac * hdr.pixdim(4);
        
    T = hdr.qoffset;
    
else
    % Arrays read as columns 
    A = transpose([hdr.srow_x(1:3), hdr.srow_y(1:3), hdr.srow_z(1:3)]);
    T = [hdr.srow_x(4); hdr.srow_y(4); hdr.srow_z(4)];
end
